package com.fs.watch.service.impl;

import com.fs.common.utils.DateUtils;
import com.fs.common.utils.StringUtils;
import com.fs.watch.domain.WatchBasicInfo;
import com.fs.watch.domain.WatchDeviceInfo;
import com.fs.watch.domain.WatchHeartRateData;
import com.fs.watch.domain.vo.AppFsUserHealthReportVo;
import com.fs.watch.domain.vo.AppHealthReportVo;
import com.fs.watch.domain.vo.GnssVo;
import com.fs.watch.domain.vo.WatchSleepDataVo;
import com.fs.watch.enums.*;
import com.fs.watch.mapper.*;
import com.fs.watch.param.WatchDataQueryParam;
import com.fs.watch.service.*;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.mybatis.spring.MyBatisSystemException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author Administrator
 * @description 针对表【watch_basic_info(设备基础数据)】的数据库操作Service实现
 * @createDate 2024-10-15 11:06:50
 */
@Service
@Slf4j
public class WatchBasicInfoServiceImpl implements WatchBasicInfoService {
    @Resource
    private WatchHeartRateDataService watchHeartRateDataService;
    @Autowired
    private WatchRriDataService watchRriDataService;
    @Autowired
    private WatchSleepDataService watchSleepDataService;
    @Autowired
    private WatchBasicInfoMapper watchBasicInfoMapper;
    @Autowired
    private WatchBloodGlucoseDataMapper watchBloodGlucoseDataMapper;
    @Autowired
    private WatchBloodPressureDataMapper watchBloodPressureDataMapper;
    @Autowired
    private WatchHeartRateDataMapper watchHeartRateDataMapper;
    @Autowired
    private WatchThirdUaDataMapper watchThirdUaDataMapper;
    @Autowired
    private WatchSpo2DataMapper watchSpo2DataMapper;
    @Autowired
    private WatchTemperatureDataMapper watchTemperatureDataMapper;
    @Autowired
    private WatchSportDataService watchSportDataService;
    @Autowired
    private WatchDeviceInfoMapper watchDeviceInfoMapper;

    /**
     * 新增数据
     *
     * @param watchBasicInfo 实例对象
     * @return 实例对象
     */
    @Override
    public Boolean insert(WatchBasicInfo watchBasicInfo) {
        //判断创建时间是否唯一
        Integer num = watchBasicInfoMapper.queryByCreateTime(watchBasicInfo.getCreateTime());
        if (num != null && num > 0) {
            return false;
        }
        try {
            watchBasicInfoMapper.insert(watchBasicInfo);
            //更新mysql的腕表数据
            WatchDeviceInfo deviceInfo = new WatchDeviceInfo();
            deviceInfo.setDeviceNumber(watchBasicInfo.getDeviceId());
            deviceInfo.setBattery(watchBasicInfo.getBattery());
            deviceInfo.setRssi(watchBasicInfo.getRssi());
            watchDeviceInfoMapper.updateByDeviceNumber(deviceInfo);
        } catch (Exception e) {
            log.error("===============watchBasicInfo:insert=========================={}", e.getMessage());
            return false;
        }
        return true;
    }


    /**
     * 通过ID查询单条数据
     *
     * @param deviceId 主键
     * @return 实例对象
     */
    @Override
    public WatchBasicInfo queryByDeviceId(String deviceId) {
//        return watchBasicInfoMapper.queryByDeviceId(deviceId);
        log.info("DeviceId: {}", deviceId);
        if (deviceId == null) {
            log.error("DeviceId is null, cannot proceed with the query.");
        }
        try {
            // 执行查询
            return watchBasicInfoMapper.queryByDeviceId(deviceId);
        } catch (Exception e) {
            log.error("Exception during query execution: ", e);
            throw new MyBatisSystemException(e);
        }


    }

    @Override
    public WatchBasicInfo queryLastByDeviceId(String deviceId) {
        return watchBasicInfoMapper.getLastByDeviceId(deviceId);
    }


    private Map<String, Object> getBasicInfoMap(WatchBasicInfo watchBasicInfo) {
        Map<String, Object> params = new HashMap<>();
        params.put("id", watchBasicInfo.getId().toString());
        params.put("deviceId", watchBasicInfo.getDeviceId());
        params.put("steps", watchBasicInfo.getSteps());
        params.put("distance", watchBasicInfo.getDistance());
        params.put("calorie", watchBasicInfo.getCalorie());
        params.put("battery", watchBasicInfo.getBattery());
        params.put("rssi", watchBasicInfo.getRssi());
        params.put("jsonDataGnssLon", watchBasicInfo.getGnssLon().toString());
        params.put("jsonDataGnssLat", watchBasicInfo.getGnssLat().toString());
        params.put("jsonDataGnssLocType", watchBasicInfo.getGnssLocType().toString());
        params.put("jsonDataCreateTime", watchBasicInfo.getGnssTime().toString());
        params.put("createTime", watchBasicInfo.getCreateTime());
        return params;
    }

    @Override
    public List<GnssVo> queryGnssByDateAndDeviceId(String date, String deviceId) {
        if (StringUtils.isBlank(date)) {
            date = DateUtils.dateTimeNow("yyyy-MM-dd");
        }
        List<WatchBasicInfo> basicInfos = watchBasicInfoMapper.queryGnssByDateAndDeviceId(date, deviceId);
        List<GnssVo> gnssVos = getGnssVos(basicInfos);

        return gnssVos;
    }

    private static @NotNull List<GnssVo> getGnssVos(List<WatchBasicInfo> basicInfos) {
        List<GnssVo> gnssVos = new ArrayList<>();
        for (WatchBasicInfo basicInfo : basicInfos) {
            List<String> gnssTimes = basicInfo.getGnssTime();
            GnssVo gnssVo = new GnssVo();
            if (gnssTimes == null || gnssTimes.isEmpty()) {
                continue;
            }
            int i = gnssTimes.size() - 1;
            gnssVo.setTime(basicInfo.getCreateTime());
            gnssVo.setLng(basicInfo.getGnssLon().get(i));
            gnssVo.setLat(basicInfo.getGnssLat().get(i));
            gnssVo.setLocType(basicInfo.getGnssLocType().get(i));
            gnssVos.add(gnssVo);
        }
        return gnssVos;
    }

    @Override
    public Map<String, Object> queryOverviewsByDateAndDeviceId(String date, String deviceId) {
        Map<String, Object> map = new HashMap<>();
        //心率
        WatchHeartRateData hrData = watchHeartRateDataService.queryByDeviceId(deviceId);
        if (hrData != null) {
            map.put("hr", hrData.getAvgBpm());
        }
        //房颤
        Map<String, Object> afData = this.watchRriDataService.queryByDateAndDeviceId(date, deviceId);
        if (afData != null && afData.get("mes") != null) {
            map.put("af", afData.get("mes").toString().split("\\.")[0]);
        }
        //步数
        //睡眠
        WatchSleepDataVo sleepData = watchSleepDataService.queryByDateAndDeviceIdAndState(date, deviceId);
        if (sleepData != null) {
            map.put("sleep", sleepData.getSleepTime());
        }

        return map;
    }

    @Override
    public List<WatchBasicInfo> page(WatchDataQueryParam param) {
        return watchBasicInfoMapper.page(param);
    }

    @Override
    public List<GnssVo> queryGnssByDate(String starTime, String endTime, String deviceId) {
        List<WatchBasicInfo> basicInfos = watchBasicInfoMapper.queryGnssByDate(starTime, endTime, deviceId);
        List<GnssVo> gnssVos = getGnssVos(basicInfos);
        return gnssVos;
    }

    /**
     * ＞3个异常 只给出3个异常，优先级顺序 血糖、血压、心率、尿酸、血氧、睡眠、体温、步数
     *
     * @param startTime
     * @param endTime
     * @param deviceId
     * @return
     */
    @Override
    public AppFsUserHealthReportVo getAppFsUserHealthReportVo(Date startTime, Date endTime, String deviceId) {
        AppFsUserHealthReportVo res = new AppFsUserHealthReportVo();
        ArrayList<AppHealthReportVo> vos = new ArrayList<>();
        AtomicInteger score = new AtomicInteger(100); //分数
        String start = DateUtils.parseDateToStr("yyyy-MM-dd hh:mm:ss", startTime);
        String end = DateUtils.parseDateToStr("yyyy-MM-dd hh:mm:ss", endTime);
        //先查询睡眠数据
        watchSleepDataService.getSleepStatus(startTime,endTime,deviceId);

        // 顺序执行各项异常处理（最多保留3个）
        getHealth(watchBloodGlucoseDataMapper.countBpByDate(start, end, deviceId), vos, WatchBgStatusEnum.class,score);
        getHealth(watchBloodPressureDataMapper.countBpByDate(start, end, deviceId), vos, WatchBpStatusEnum.class,score);
        getHealth(watchHeartRateDataMapper.countBpByDate(start, end, deviceId), vos, WatchHrStatusEnum.class,score);
        getHealth(watchThirdUaDataMapper.countUaByDate(start, end, deviceId), vos, WatchUaStatusEnum.class,score);
        getHealth(watchSpo2DataMapper.countSpByDate(start, end, deviceId), vos, WatchSpo2StatusEnum.class,score);
        getHealth(watchSleepDataService.countSlByDate(start, end, deviceId), vos, WatchSleepStatusEnum.class,score);
        getHealth(watchTemperatureDataMapper.countByDate(start, end, deviceId), vos, WatchTemperatureStatusEnum.class,score);


        // 2. 步数（枚举可能不需要，可直接判断）
        WatchStepStatusEnum weekStatus = watchSportDataService.getWeekStatus(start, end, deviceId);
        if (!Objects.equals(weekStatus.getValue(), WatchStepStatusEnum.NORMAL.getValue())){
            AppHealthReportVo vo = new AppHealthReportVo();
            vo.setPrediction(weekStatus.getPrediction());
            vo.setAnalysis(weekStatus.getAnalysis());
            vo.setSuggestion(weekStatus.getSuggestion());
            vo.setPrecautions(weekStatus.getPrecautions());
            vos.add(vo);
            score.addAndGet(-5); // 异常就减分;
        }
        // 3. 默认健康报告
        if (vos.isEmpty()) {
            AppHealthReportVo normalVo = new AppHealthReportVo();
            normalVo.setPrediction("您本周所有监测指标均在正常范围内，暂未发现异常风险。");
            normalVo.setAnalysis("当前数据表明身体各系统功能处于平衡状态，是健康生活方式的积极体现。");
            normalVo.setSuggestion("继续保持现有健康习惯；建议每3-6个月定期监测关键指标（如血压、血糖）。");
            normalVo.setPrecautions("均衡饮食（每日摄入12种以上食物）；每周150分钟中等强度运动；保证7小时睡眠。");
            vos.add(normalVo);
        }

        res.setList(vos.size() > 3 ? vos.subList(0, 3) : vos);
        int finalScore = Math.max(score.get(), 0);
        res.setScore(finalScore);

        WatchHealthEnum healthEnum = null;
        if(finalScore>=90){
            healthEnum = WatchHealthEnum.EXCELLENT;
        } else if (finalScore>=80){
            healthEnum = WatchHealthEnum.GOOD;
        } else if (finalScore>=70){
            healthEnum = WatchHealthEnum.GENERAL;
        } else{
            healthEnum = WatchHealthEnum.POOR;
        }
        res.setTitle(healthEnum.getTitle());
        res.setDesc(healthEnum.getDesc());

        return res;
    }




    public <T extends Enum<T> & HealthStatusEnum> void getHealth(
            List<Map<String, Object>> data,
            List<AppHealthReportVo> vos,
            Class<T> enumClass,
            AtomicInteger  score
    ) {
        if (data == null) {
            return;
        }
        for (Map<String, Object> map : data) {
            Integer status = Integer.valueOf(map.get("status").toString());
            Integer count = Integer.valueOf(map.get("count").toString());
            if (count > 0) {
                AppHealthReportVo vo = createHealthReportVo(status, count, enumClass,score);
                if (vo != null) {
                    vos.add(vo);
                }
            }
        }
    }

    public <T extends Enum<T> & HealthStatusEnum> AppHealthReportVo createHealthReportVo(
            Integer status,
            Integer count,
            Class<T> enumClass,
            AtomicInteger  score
    ) {
        if (status == null || enumClass == null) return null;

        T matchedEnum = Arrays.stream(enumClass.getEnumConstants())
                .filter(e -> Objects.equals(e.getValue(), status))
                .findFirst()
                .orElse(null);

        if (matchedEnum == null || matchedEnum.getPrediction() == null) return null;

        AppHealthReportVo vo = new AppHealthReportVo();
        vo.setPrediction(count != null ? String.format(matchedEnum.getPrediction(), count) : matchedEnum.getPrediction());
        vo.setAnalysis(matchedEnum.getAnalysis());
        vo.setSuggestion(matchedEnum.getSuggestion());
        vo.setPrecautions(matchedEnum.getPrecautions());
        //扣得分 出现一次异常减5分，无数据不扣分
        score.addAndGet(-(5*count)); // 异常就减分
        return vo;
    }


}
