package com.fs.watch.service.impl;

import com.fs.watch.domain.WatchDeviceInfo;
import com.fs.watch.domain.WatchDeviceStatus;
import com.fs.watch.enums.WatchDeviceStatusEnum;
import com.fs.watch.mapper.WatchDeviceInfoMapper;
import com.fs.watch.mapper.WatchDeviceStatusMapper;
import com.fs.watch.service.WatchDeviceStatusService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 设备原始回传信息记录(WatchDeviceStatus)表服务实现类
 *
 * @author makejava
 * @since 2024-12-17 15:11:06
 */
@Service("watchDeviceStatusService")
@Slf4j
public class WatchDeviceStatusServiceImpl implements WatchDeviceStatusService {
    @Autowired
    private WatchDeviceStatusMapper watchDeviceStatusMapper;
    @Autowired
    private WatchDeviceInfoMapper watchDeviceInfoMapper;


    /**
     * 新增数据
     *
     * @param watchDeviceStatus 实例对象
     * @return 实例对象
     */
    @Override
    public Boolean insert(WatchDeviceStatus watchDeviceStatus) {
        Integer num = watchDeviceStatusMapper.queryByCreateTime(watchDeviceStatus.getEventtime());
        if (num != null && num > 0) {
            return false;
        }
        try {
            watchDeviceStatusMapper.insert(watchDeviceStatus);
            //更新mysql的腕表数据
            WatchDeviceInfo deviceInfo = new WatchDeviceInfo();
            deviceInfo.setDeviceNumber(watchDeviceStatus.getDeviceId());
            deviceInfo.setStatus(WatchDeviceStatusEnum.toType(watchDeviceStatus.getStatus()).getValue());
            watchDeviceInfoMapper.updateByDeviceNumber(deviceInfo);
        } catch (Exception e) {
            log.error(e.getMessage());
            return false;
        }
        return true;
    }

    @Override
    public WatchDeviceStatus getLastByDeviceId(String deviceId) {
        return watchDeviceStatusMapper.getLastByDeviceId(deviceId);
    }

}
